#!/usr/bin/env python3

from __future__ import print_function

"""Calculate magnetic deviation from a table"""

"""
RCSid:
	$Id: deviation.py,v 1.3 2022/05/05 18:31:52 sjg Exp $

	@(#) Copyright (c) 2019, Simon J. Gerraty

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net

"""

class Deviation(object):
    """We are initialized with a list of deviation
    readings.  Typically representing 30 or 15 degree
    increments.
    Negative are West error
    Positive are East error.

    """
    
    def __init__(self, deviation=None):
        self.deviation_set(deviation)

    def deviation_set(self, deviation):
        self.deviation = deviation
        self.n = len(deviation)
        if self.n > 0:
            self.interval = int(360/self.n)
        else:
            self.interval = 1

    def compass(self, m):
        """magnetic to compass - TVMDCAW"""

        d = self.lookup(m)
        return m - d

    def magnetic(self, c):
        """compass to magnetic - CDMVTAE"""

        d = self.lookup(c)
        return c + d

    def lookup(self, b):
        x = int((b / self.interval) % self.n)
        z = b % self.interval
        d = self.deviation[x]
        if z > 0.0:
            # we need to interpolate
            delta = (self.deviation[int((x+1) % self.n)] - self.deviation[x]) \
                    * z / self.interval
            d += delta
        return d

if __name__ == '__main__':
    import sys
    import getopt

    opts,args = getopt.getopt(sys.argv[1:], 'df:u')

    debug = 0
    correcting = True
    deviation = []
    for o,a in opts:
        if o == '-d':
            debug += 1
        elif o == '-f':
            for line in open(a).readlines():
                for x in line.strip().split(','):
                    deviation.append(float(x))
        elif o == '-u':
            correcting = False

    if len(deviation) == 0:
        deviation = [1,2,3,2,1,0,-1,-2,-3,-2,-1,0]
    dev = Deviation(deviation)
    for a in args:
        m = float(a)
        c = dev.compass(m)
        d = dev.lookup(m)
        e = dev.lookup(c)
        n = dev.magnetic(c)
        print('{:03.0f} M ({:03.0f}) -> {:03.0f} C ({:03.0f}) -> {:03.0f} M'.format(m, d, c, e, n))

        
